@@ -20,28 +20,57 @@ spatmca <- function(x1, x2, Y1, Y2, M = 5, K = NULL, K.select = ifelse(is.null(K
   }
   n = nrow(Y1)
   stra <- sample(rep(1:M, length.out = nrow(Y1)))
+  
+  tempegvl1 <- svd(Y1/n)
+  tempegvl2 <- svd(Y2/n)
+  dd <- t(Y1)%*%Y2/n
+  tempegvl3 <- svd(dd)
+  egvl1 <- tempegvl1$d[1]
+  egvl2 <- tempegvl2$d[1]
+  egvl3 <- tempegvl3$d[1]
+  
   if(is.null(tau2u)&&is.null(tau2v)){
-    tau2u = tau2v = 0
-    ntau2u <- 1
-    ntau2v <- 1
+    ntau2u <- ntau2v <- 11
+    
+    indexu <- sort(abs(tempegvl3$u[,1]),decreasing=T,index.return=T)$ix
+    nu1u <- indexu[2]
+    nu2u <- indexu[ncol(Y1)]
+    max.tau2u <- 2*abs(dd[nu1u,]%*%tempegvl3$v[,1])
+    min.tau2u <- abs(dd[nu2u,]%*%tempegvl3$v[,1])
+    tau2u <- c(0,exp(seq(log(min.tau2u), log(max.tau2u), length = (ntau2u-1)))) 
+  
+    indexv <- sort(abs(tempegvl3$v[,1]),decreasing=T,index.return=T)$ix
+    nu1v <- indexv[2]
+    nu2v <- indexv[ncol(Y2)]
+    max.tau2v <- 2*abs(t(dd)[nu1v,]%*%tempegvl3$u[,1])
+    min.tau2v <- abs(t(dd)[nu2v,]%*%tempegvl3$u[,1])
+    tau2v <- c(0,exp(seq(log(min.tau2v), log(max.tau2v), length = (ntau2v-1)))) 
+    
   }else if (is.null(tau2u)){ 
-    tau2u <- 0
-    ntau2u <- 1
+    ntau2u <- 11
+    indexu <- sort(abs(tempegvl3$u[,1]),decreasing=T,index.return=T)$ix
+    nu1u <- indexu[2]
+    nu2u <- indexu[ncol(Y1)]
+    max.tau2u <- 2*abs(dd[nu1u,]%*%tempegvl3$v[,1])
+    min.tau2u <- abs(dd[nu2u,]%*%tempegvl3$v[,1])
+    tau2u <- c(0,exp(seq(log(min.tau2u), log(max.tau2u), length = (ntau2u-1)))) 
+    
     ntau2v <- length(tau2v)
   }else if (is.null(tau2v)){ 
-    tau2v <- 0
+    ntau2v <- 11
+    indexv <- sort(abs(tempegvl3$v[,1]),decreasing=T,index.return=T)$ix
+    nu1v <- indexv[2]
+    nu2v <- indexv[ncol(Y2)]
+    max.tau2v <- egvl3*abs(t(dd)[nu1v,]%*%tempegvl3$u[,1])
+    min.tau2v <- egvl3*abs(t(dd)[nu2v,]%*%tempegvl3$u[,1])
+    tau2v <- c(0,exp(seq(log(min.tau2v), log(max.tau2v), length = (ntau2v-1)))) 
+    
     ntau2u <- length(tau2u)
-    ntau2v <- 1
   }else{
     ntau2u <- length(tau2u)
     ntau2v <- length(tau2v)
   }
-  tempegvl1 <- svd(Y1/n)
-  tempegvl2 <- svd(Y2/n)
-  tempegvl3 <- svd(t(Y1)%*%Y2/n)
-  egvl1 <- tempegvl1$d[1]
-  egvl2 <- tempegvl2$d[1]
-  egvl3 <- tempegvl3$d[1]
+  
   
   if(is.null(tau1u) && is.null(tau1v)) {
     ntau1u <- 11
